function sample = RandomSample(n, type, params)
% generating random number specified by the type and parameters
% n is the number of samples required
% type is the distribution type
% params is the cell array containing corresponding paramemters
% output: sample is the matrix n-by-dim

% multivariate normal distribution
if type == 1
    sample = mvnrnd(repmat(params{1}, n, 1), params{2});
    return;
end

% multivariate t-distribution
if type == 2
    sample = mvnrnd(repmat(params{1}, n, 1), params{2});
    denom = chi2rnd(params{3}, n, 1); % params{3} is the degree of freedom
    denom = sqrt(denom/params{3});
    sample = sample./repmat(denom, 1, size(sample, 2));
    return;
end

% multivariate gamma distribution
if type == 3
    sample = zeros(n, length(params{1}));
    for i = 1 : n
        tmp = mvnrnd(repmat(params{1}, params{3}, 1), params{2});
        sample(i,:) = sum(tmp.^2)/2;
    end
    return;
end

% mixed t and chi^2 distribution
if type == 4
   idx = floor(params{1}/2);
   p1 = trnd(params{2}, n, idx);
   p2 = chi2rnd(params{3}, n, params{1}-idx);
   sample = [p1, p2];
   return;
end
