#include "mex.h"
#include <time.h>
#include <stdlib.h>
#include <matrix.h>
#include <cmath>
#include <algorithm>
// using namespace std;
// extern void _main();

/*************************************************************************************/
void randperm(int size, int *result)
{
  int r = 0;
  int tmp = 0;
  int i = 0;

  for(i = (size-1); i > 0; i--) {
    r = rand() % (i+1);

    tmp = result[i];
    result[i] = result[r];
    result[r] = tmp;
  }
  
  return;
}

/*************************************************************************************/
/* permutation test conditional on previous conformity */
bool permutationTest(const double *rank, const double *climit, 
		     int m0, int num, int dim, int nbound, int minwin, int maxwin,
		     double *Tval, const double *mu, const double *sd, double lambda)
{
  int *sequence;
  int i, j, k, win, pwin;
  int icount = 0;
  int lower = m0;
  bool flag = true;
  double pTval, colTval;
  double *tmpRank, *pRank;

  /* initialize the permutation index */
  sequence = (int *)mxCalloc(num, sizeof(int));
  if (sequence == NULL) {
    return 1;
  } else {
    for (i=0; i<num; i++) {
      sequence[i] = i;
    }
  }

  /* copy the rank to tmpRank */
  pRank = (double *)mxCalloc(num*dim, sizeof(double));
  tmpRank = (double *)mxCalloc(num*dim, sizeof(double));
  if ((pRank == NULL) || (tmpRank == NULL)) {
    return false;
  } 
  
  win = std::min(std::max(num-m0,minwin), maxwin);
  /* generate permutation */
  if (lower < (num-win)) {
    lower = num - win;
  }

  while (icount < nbound) {
    randperm(num, sequence);
    flag = true;

    /* assign the permutated values */
    for (i=0; i<dim; i++) {
      for (j=0; j<num; j++) {
	pRank[i*num + j] = rank[i*num + sequence[j]];
	tmpRank[i*num + j] = pRank[i*num + j];
      }
    }
    
    /* permutation test for previous indices */
    for (i=1; i<(num-lower); i++) {
      pTval = 0;
      pwin = std::min(std::max(num-m0-i,minwin), maxwin);

      for (j=0; j<dim; j++) {
	colTval = 0;

	for (k=lower-pwin+1; k<num-i; k++) {
	  if (pRank[j*num + k] > pRank[j*num + num-i])
	    tmpRank[j*num + k]--;

	  if (k>=num-i-pwin)
	    colTval = (1-lambda)*colTval + tmpRank[j*num + k] - (num-i+1)/2;
	}
	
	colTval = std::pow(colTval/sd[num-m0-i-1], 2);
	
	pTval += colTval;
      }

      /* check if the value is within the control limit */
      if (pTval >= climit[num-m0-i-1]) {
	flag = false;
	break;
      }
    }

    /* include this permutation if sequences pass */
    if (flag) {
      Tval[icount] = 0;

      for (j=0; j<dim; j++) {
	colTval = 0;

	for (k=num-win; k<num; k++) {
	  colTval = colTval*(1-lambda) + pRank[j*num + k] - (num+1)/2;
	}

	colTval = std::pow(colTval/sd[num-m0-1], 2);
	Tval[icount] += colTval;
      }

      icount++;
    }

  }

  mxFree(sequence);
  mxFree(pRank);
  mxFree(tmpRank);
  return true;
}


/*************************************************************************************/
/*
input: original rank matrix, required permutation number, control limit at previous steps, alpha error
output: cut off value at current sample
*/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  int nbound, dim, num, m0, minwin, maxwin;
  double *cutoff, *rankMat, *climit, *mu, *sd;
  double *Tval;
  double lambda;
  bool flag = true;

  if ((nlhs != 1) || (nrhs != 9)) {
    mexPrintf("The number of argument and returning value is wrong!\n");
  }

  rankMat = mxGetPr(prhs[0]);        
  nbound = (int)mxGetScalar(prhs[1]);
  dim = (int)mxGetN(prhs[0]);        
  num = (int)mxGetM(prhs[0]);        
  climit = mxGetPr(prhs[2]);         
  m0 = (int)mxGetScalar(prhs[3]);    
  minwin = (int)mxGetScalar(prhs[4]);
  maxwin = (int)mxGetScalar(prhs[5]);
  mu = mxGetPr(prhs[6]);         
  sd = mxGetPr(prhs[7]);         
  lambda = (double)mxGetScalar(prhs[8]);


  plhs[0] = mxCreateDoubleMatrix(1, nbound, mxREAL); 
  Tval = mxGetPr(plhs[0]);

  if (Tval == NULL) {
    mexPrintf("Error in allocation memory!\n");
    return;
  }

  srand(time(NULL));

  flag = permutationTest(rankMat, climit, m0, num, dim, nbound, minwin, maxwin, Tval, mu, sd, lambda);
  if (!flag) {
    mexPrintf("error in permutation test\n");
  }

  return;
}

